#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include <commctrl.h>
#include <string.h>
#include "Drawres.h"
#include "main.h"

#ifdef __BORLANDC__
#define UNUSED(x)
#else
#define UNUSED(x) x = x
#endif

#define NORMAL_ZOOM 10

static HINSTANCE hInst;      // Instance handle
HWND hwndMain;               // Main window handle
static HBITMAP BitmapHandle;
static LRESULT CALLBACK MainWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
static UINT ImageWidth, ImageHeight;
static HBITMAP LoadBMP(const char *FileName, UINT *width, UINT *height);
static void ShowImage(HDC dc);
int Zoom = NORMAL_ZOOM;
int YOffset;
int XOffset;
const char *CmdLine;

int GetDispWidth(void)
{
  return ImageWidth*Zoom/NORMAL_ZOOM;
}

int GetDispHeight(void)
{
  return ImageHeight*Zoom/NORMAL_ZOOM;
}

static BOOL InitApplication(void)
{
  WNDCLASS wc;
  InitCommonControls();
  memset(&wc, 0, sizeof(WNDCLASS));
  wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
  wc.lpfnWndProc = (WNDPROC)MainWndProc;
  wc.hInstance = hInst;
  wc.hbrBackground = (HBRUSH )(COLOR_WINDOW + 1);
  wc.lpszClassName = "DrawWndClass";
  wc.lpszMenuName = MAKEINTRESOURCE(IDMAINMENU);
  wc.hCursor = LoadCursor(NULL,IDC_ARROW);
  wc.hIcon = LoadIcon(NULL,IDI_APPLICATION);
  if (!RegisterClass(&wc))
    return 0;

  return 1;
}

void Open(void)
{
  static char Filter[] = "Bitmaps (*.bmp)\0*.bmp\0All (*.*)\0*.*\0";
  char FileName[256] = "";

  OPENFILENAME OpenFilename;

  memset(&OpenFilename, 0, sizeof(OpenFilename));
  OpenFilename.lStructSize = sizeof(OpenFilename);
  OpenFilename.hwndOwner = hwndMain;
  OpenFilename.hInstance = hInst;
  OpenFilename.lpstrFilter = Filter;
  OpenFilename.lpstrFile = FileName;
  OpenFilename.nMaxFile = sizeof(FileName);
  OpenFilename.Flags = OFN_FILEMUSTEXIST;

  if(GetOpenFileName(&OpenFilename))
  {
    if(BitmapHandle)
      DeleteObject(BitmapHandle);

    BitmapHandle = LoadBMP(FileName, &ImageWidth, &ImageHeight);
    InvalidateRect(hwndMain, 0, TRUE); // Will erase background, and paint image
  }
}

void MainWndProc_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  switch(id)
  {
    case IDM_EXIT:
      PostMessage(hwnd, WM_CLOSE, 0, 0);
      break;
    case IDM_OPEN:
      Open();
      break;
    case IDM_ZOOM_IN:
      {
        if(Zoom < 100 && BitmapHandle)
        {
          if(Zoom < 2*NORMAL_ZOOM)
            Zoom++;
          else
            Zoom += Zoom/10;
          SetScroll();
          InvalidateRect(hwnd, 0, TRUE); // Will erase background, and paint image
        }
      }
      break;
    case IDM_ZOOM_OUT:
      {
        if(Zoom > 1 && BitmapHandle)
        {
          if(Zoom > 2*NORMAL_ZOOM)
            Zoom -= Zoom/10;
          else
            Zoom--;
          SetScroll();
          InvalidateRect(hwnd, 0, TRUE); // Will erase background, and paint image
        }
      }
      break;
    case IDM_SCROLL_LINE_DOWN:
      VScroll(SB_LINEDOWN, 0);
      break;
    case IDM_SCROLL_LINE_UP:
      VScroll(SB_LINEUP, 0);
      break;
    case IDM_SCROLL_LINE_LEFT:
      HScroll(SB_LINELEFT, 0);
      break;
    case IDM_SCROLL_LINE_RIGHT:
      HScroll(SB_LINERIGHT, 0);
      break;

    case IDM_ABOUT:
      MessageBox(hwnd, "Bitmap Image Viewer\r\nNo copyright", "Draw", MB_OK);
      break;
  }
  UNUSED(hwndCtl);
  UNUSED(codeNotify);
}

int Paint(void)
{
  PAINTSTRUCT PaintStruct;
  RECT Rect, ClientRect;
  HBRUSH BackBrush;
  HDC DCHandle = BeginPaint(hwndMain, &PaintStruct);
  if(BitmapHandle)
    ShowImage(DCHandle);
  /* Erase space to the right and at the buttom */
  GetClientRect(hwndMain, &ClientRect);
  BackBrush = CreateSolidBrush(RGB(255, 255, 255));

  if(GetDispWidth() < ClientRect.right)
  {
    Rect.left = GetDispWidth();
    Rect.right = ClientRect.right;
    Rect.top = 0;
    Rect.bottom = ClientRect.bottom < GetDispHeight() ? ClientRect.bottom : GetDispHeight();
    FillRect(DCHandle, &Rect, BackBrush);
  }

  if(GetDispHeight() < ClientRect.bottom)
  {
    Rect.left = 0;
    Rect.right = ClientRect.right;
    Rect.top = GetDispHeight();
    Rect.bottom = ClientRect.bottom;
    FillRect(DCHandle, &Rect, BackBrush);
  }
  DeleteObject(BackBrush);
  EndPaint(hwndMain, &PaintStruct);

  return TRUE;
}

static LRESULT CALLBACK MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_COMMAND:
      HANDLE_WM_COMMAND(hwnd, wParam, lParam, MainWndProc_OnCommand);
      return TRUE;
    case WM_DESTROY:
      PostQuitMessage(0);
      return TRUE;
    case WM_CREATE:
      if(CmdLine)
        BitmapHandle = LoadBMP(CmdLine, &ImageWidth, &ImageHeight);
      break;
    case WM_PAINT:
      if(Paint())
        return TRUE;
      break;
    case WM_SIZE:
      SetScroll();
      break;
    case WM_VSCROLL:
      VScroll(LOWORD(wParam), HIWORD(wParam));
      break;
    case WM_HSCROLL:
      HScroll(LOWORD(wParam), HIWORD(wParam));
      break;
    case WM_ERASEBKGND:
      return TRUE;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nCmdShow)
{
  MSG msg;
  HANDLE hAccelTable;

  hInst = hInstance;

  if (!InitApplication())
    return 0;
  if(lpCmdLine && *lpCmdLine)
    CmdLine = lpCmdLine;

  hAccelTable = LoadAccelerators(hInst, MAKEINTRESOURCE(IDACCEL));

  hwndMain = CreateWindow("DrawWndClass", "Draw",
                          WS_MINIMIZEBOX | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_MAXIMIZEBOX | WS_CAPTION | WS_BORDER | WS_SYSMENU | WS_THICKFRAME,
                          CW_USEDEFAULT, 0 , CW_USEDEFAULT, 0,
                          NULL,
                          NULL,
                          hInst,
                          NULL);

  if (hwndMain == 0)
    return 0;

  ShowWindow(hwndMain, SW_SHOW);
  while(GetMessage(&msg,NULL,0,0))
  {
    if(!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  }
  UNUSED(hPrevInstance);
  UNUSED(lpCmdLine);
  UNUSED(nCmdShow);
  return msg.wParam;
}

static HBITMAP LoadBMP(const char *FileName, UINT *Width, UINT *Height)
{
  HBITMAP Bitmap;
  BITMAP BitMapInfo;
  char Title[1024];
  Bitmap = LoadImage(0, FileName, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
  if(!Bitmap)
  {
    char msg[1024];
    sprintf(msg, "Failed to load bitmap\r\n %s", FileName);
    MessageBox(hwndMain, msg, "Draw", MB_OK);
    return 0;
  }
  GetObject(Bitmap, sizeof(BITMAP), &BitMapInfo);
  *Width = BitMapInfo.bmWidth;
  *Height = BitMapInfo.bmHeight;
  Zoom = NORMAL_ZOOM;
  SetScroll();
  sprintf(Title, "Draw [%s]", FileName);
  SetWindowText(hwndMain, Title);
  return Bitmap;
}

static void ShowImage(HDC dc)
{
  HDC MemDC;
  HBITMAP OldBitmap;
  MemDC = CreateCompatibleDC(dc);
  OldBitmap = SelectObject(MemDC, BitmapHandle);
  SetStretchBltMode(dc, HALFTONE);
  StretchBlt(dc, -XOffset, -YOffset, GetDispWidth(), GetDispHeight(), MemDC, 0, 0, ImageWidth, ImageHeight, SRCCOPY);
  SelectObject(MemDC, OldBitmap);
  DeleteObject(MemDC);
}

